<?php

class Aluno extends ControladorUsuario{

    public function __construct($id) {
        parent::__construct();
        parent::setUserId($id);
    }
   
    public function desvincularTurma(){
        $connection = parent::getConnection();
        $ID = (int)parent::getUserId();
        $resultado = false;
        try {
            $stmt = $connection->prepare('UPDATE aluno SET idTurma=NULL WHERE alu_id=:iduser');
            $stmt->bindValue(':iduser', $ID, PDO::PARAM_INT);
            $stmt->execute();
            $resultado = true;
        } catch(PDOException $e) {
            echo '<br>'.$e.'<br>';
        }
        
        return $resultado;
    }
    
    public function listarArquivosDaAtividade($idatv){
        $lista = null;
        try{
            $connection = parent::getConnection();
            $stmt = $connection->prepare('SELECT * FROM arquivo WHERE arquivo.idatividade=:idatv AND arquivo.idusuario=:iduser');
            $ID = (int)parent::getUserId();
            $stmt->bindParam(':iduser', $ID, PDO::PARAM_INT);
            $stmt->bindParam(':idatv', $idatv, PDO::PARAM_INT);

            $stmt->execute();
            $lista = array();
            $cont = 0;

            while ($result = $stmt->fetch(PDO::FETCH_ASSOC)){
                $lista[$cont] = new Arquivo($result);
                $cont++;
            }
            
        }  catch (Exception $e){
            echo $e->getMessage();
        }
        
        return $lista;
    }
    
    public function solicitarEntradaEmTurma($idturma){
        $connection = parent::getConnection();
        $ID = (int)parent::getUserId();
        $resultado = false;
        try {
            $stmt = $connection->prepare('call solicitarEntradaEmTurma(:iduser,:idturma)');
            $stmt->bindValue(':iduser', $ID, PDO::PARAM_INT);
            $stmt->bindValue(':idturma', $idturma, PDO::PARAM_INT);
            $stmt->execute();
            $result = $stmt->fetch(PDO::FETCH_ASSOC);
            $resultado = ($result['sucess']==='1');
            
        } catch(PDOException $e) {
            echo '<br>'.$e.'<br>';
        }
        
        return $resultado;
    }
    
    public function cancelarSolicitacao(){
        $resultado = false;
        $solicitacao = $this->getSolicitacao();
        if($solicitacao)
        {
            $id = (int)$solicitacao['id'];
            $connection = parent::getConnection();
            try {
                $stmt = $connection->prepare('DELETE FROM solicitacao WHERE solicitacao.idsolicitacao=:id;');
                $stmt->bindValue(':id', $id, PDO::PARAM_INT);
                $stmt->execute();
                $resultado = true;
            } catch(PDOException $e) {
                echo '<br>'.$e.'<br>';
            }
        }
        
        return $resultado;
        
    }
    
    public function getSolicitacao(){
        $connection = parent::getConnection();
        $ID = (int)parent::getUserId();
        $resultado = false;
        try {
            $stmt = $connection->prepare('select * from instituicao,curso,disciplina,turma,solicitacao  WHERE disciplina.disc_id=turma.disc_id AND turma.idTurma=solicitacao.Turma_idTurma AND solicitacao.Aluno_alu_id=:iduser AND curso.cur_id=disciplina.cur_id AND instituicao.inst_id=curso.inst_id');
            $stmt->bindValue(':iduser', $ID, PDO::PARAM_INT);
            $stmt->execute();
            $result = $stmt->fetch(PDO::FETCH_ASSOC);

            if($result)
            {
                $resultado = array(
                    'disciplina_nome'=>$result['disc_nome'],
                    'instituicao_nome'=>$result['inst_nome'],
                    'instituicao_uf'=>$result['inst_uf'],
                    'curso_nome'=>$result['cur_nome'],
                    'curso_sigla'=>$result['cur_sigla'],
                    'semestre'=>$result['tur_semestre'],
                    'ano'=>$result['tur_ano'],
                    'id'=>$result['idsolicitacao']);
            }
        } catch(PDOException $e) {
            echo '<br>'.$e.'<br>';
        }
        
        return $resultado;
    }
    
    public function getTurma(){
        $connection = parent::getConnection();
        $ID = (int)parent::getUserId();
        $stmt = $connection->prepare('Select * from turma where `idTurma` = ( Select `idTurma` from aluno where alu_id =:id )');
        $stmt->bindValue(':id', $ID, PDO::PARAM_INT);
        $execute = $stmt->execute();
        if($execute){
            $result = $stmt->fetch(PDO::FETCH_ASSOC);
            if($result){
                return new Turma($result);
            }
        } else {
            return null;
        }

    }
    

    
}
