<?php

/**
 * Description of ControladorUsuario
 */
use config\Database;

class ControladorUsuario {

    private $connection;
    protected $iduser = -1;
    private $nickname = null;
    private $firstname = null;
    private $lastname = null;
    private $email = null;
    private $datacadastro = null;

    public function getConnection() {
        return $this->connection;
    }

    public function __construct() {
        try {
            $this->connection = new PDO(
                    'mysql:host=localhost;dbname=' . Database::$mysql_dbname, Database::$mysql_user, Database::$mysql_password
            );
        } catch (PDOException $e) {
            echo '<code>' . $e->getMessage() . '</code>';
            die();
        }
    }

    public function changePassword($new) {

        if($this->iduser != -1){
            try {
                $stmt = $this->connection->prepare("UPDATE usuario SET senha = MD5(:new) WHERE usu_id = :id");
                $stmt->bindParam(':new', $new, PDO::PARAM_STR);
                $stmt->bindParam(':id', $this->iduser, PDO::PARAM_INT);
                return $stmt->execute();
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function checkEmail($email) {
        try {
            $stmt = $this->connection->prepare('SELECT * FROM usuario WHERE email=:email');
            $stmt->bindParam(':email', $email, PDO::PARAM_INT);
            return $stmt->execute() && $stmt->rowCount() != 0;
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return null;
    }

    public function getNameEstrutura($id) {
        try {
            $stmt = $this->connection->prepare('SELECT ed_nome FROM estrutura_de_dado WHERE ed_id=:id');
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            if ($stmt->execute() && $stmt->rowCount() == 1) {
                $dado = $stmt->fetch(PDO::FETCH_ASSOC);
                return $dado['ed_nome'];
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return null;
    }

    public function getArquivo($id) {
        try {
            $stmt = $this->connection->prepare('SELECT * FROM arquivo WHERE arq_id=:id');
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            if ($stmt->execute() && $stmt->rowCount() == 1) {
                return new Arquivo($stmt->fetch(PDO::FETCH_ASSOC));
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return null;
    }

    public function setUserId($id) {
        $this->iduser = $id;
        try {
            $stmt = $this->connection->prepare('select * FROM usuario WHERE usu_id=:id;');
            $stmt->bindValue(':id', $id, PDO::PARAM_INT);
            if ($stmt->execute() && $stmt->rowCount() == 1) {
                $result = $stmt->fetch(PDO::FETCH_ASSOC);
                $this->firstname = $result['usu_fname'];
                $this->lastname = $result['usu_lname'];
                $this->nickname = $result['usu_nickname'];
                $this->datacadastro = $result['dia_cadastro'];
                $this->email = $result['email'];
                return true;
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return false;
    }

    public function getUserId() {
        return $this->iduser;
    }

    public function checkAcess($email, $password) {
        try {
            $stmt = $this->connection->prepare('call CheckPassword(:email, :password);');
            $stmt->bindValue(':email', $email, PDO::PARAM_STR);
            $stmt->bindValue(':password', $password, PDO::PARAM_STR);
            if ($stmt->execute() && $stmt->rowCount() == 1) {
                $dado = $stmt->fetch(PDO::FETCH_ASSOC);
                return $dado['sucess'];
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return false;
    }

    public function setUser($email) {
        try {
            $stmt = $this->connection->prepare('select * FROM usuario WHERE email=:email;');
            $stmt->bindValue(':email', $email, PDO::PARAM_STR);
            if($stmt->execute() && $stmt->rowCount() == 1){
                $result = $stmt->fetch(PDO::FETCH_ASSOC);
                $this->iduser = $result['usu_id'];
                $this->firstname = $result['usu_fname'];
                $this->lastname = $result['usu_lname'];
                $this->nickname = $result['usu_nickname'];
                $this->datacadastro = $result['dia_cadastro'];
                return true;
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return false;
    }

    public function getTitleNameBy($firstname, $lastname, $nickname, $email) {

        if ($nickname != null && $nickname != '') {
            $title = $nickname;
        } else if ($firstname != null && $firstname != '') {
            $title = $firstname;
        } else if ($lastname != null && $lastname != '') {
            $title = "Sr(a). " . $lastname;
        } else {
            $title = $email;
        }
        return $title;
    }

    public function getTitleName() {
        return $this->getTitleNameBy($this->firstname, $this->lastname, $this->nickname, $this->email);
    }

    public function setNickname($nickname) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('call setNickname(:iduser, :nickname);');
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':nickname', $nickname, PDO::PARAM_STR);
                if($stmt->execute() && $stmt->rowCount() == 1){
                    $dado = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $dado['sucess'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function setFirstName($firstname) {

        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('call setFname(:iduser, :fname);');
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':fname', $firstname, PDO::PARAM_STR);
                if($stmt->execute() && $stmt->rowCount() == 1){
                    $dado = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $dado['sucess'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function setLastName($lastname) {

        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('call setLname(:iduser, :lname);');
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':lname', $lastname, PDO::PARAM_STR);
                if($stmt->execute() && $stmt->rowCount() == 1){
                    $dado = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $dado['sucess'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function setImage($path) {

        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('call setImgUser(:iduser, :path);');
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':path', $path, PDO::PARAM_STR);
                if($stmt->execute() && $stmt->rowCount() == 1){
                    $dado = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $dado['sucess'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function updateTopico($id, $titulo, $texto, $particular, $report, $anonimo, $idturma) {
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare('UPDATE topico SET `Turma_idTurma`=:idturma,`visible_only_by_class`=:particular,`report`=:report,`anonimo`=:anonimo,`top_assunto`=:title, `topico_texto`=:texto, `data`=current_date(), `hora`=current_time() WHERE `top_id`=:id AND `usu_id`=:iduser;');

                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':id', $id, PDO::PARAM_INT);
                $stmt->bindParam(':title', $titulo, PDO::PARAM_STR);
                $stmt->bindParam(':texto', $texto, PDO::PARAM_STR);
                $stmt->bindParam(':idturma', $idturma, $idturma != null ? PDO::PARAM_INT : PDO::PARAM_NULL);
                $stmt->bindParam(':particular', $particular, PDO::PARAM_BOOL);
                $stmt->bindParam(':report', $report, PDO::PARAM_BOOL);
                $stmt->bindParam(':anonimo', $anonimo, PDO::PARAM_BOOL);
                return $stmt->execute();
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function deleteTopico($id) {
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare('DELETE FROM topico WHERE `top_id`=:id AND `usu_id`=:iduser');
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':id', $id, PDO::PARAM_INT);
                return $stmt->execute();
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function listarRespostasDoUsuario() {
        try {
            $stmt = $this->connection->prepare("SELECT * FROM resposta,topico,usuario,configuracoes WHERE configuracoes.Usuario_usu_id = topico.usu_id and usuario.usu_id = topico.usu_id and topico.top_id = resposta.top_id and resposta.usu_id=:id ORDER BY topico.top_id, resposta.`data`, resposta.`hora`");
            $stmt->bindParam(':id', $this->iduser, PDO::PARAM_INT);

            if ($stmt->execute()) {
                $saida = array();
                $i = 0;
                $j = -1;
                $ant = null;
                while ($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
                    if ($ant == null || $result['top_id'] != $ant['top_id']) {
                        $ant = $result;
                        $j++;
                        $i = 0;
                        $saida[$j] = array();
                    }
                    $saida[$j][$i] = new Resposta($result);
                    $saida[$j][$i]->setVotosPositivos($this->getTotalVotosPositivos($saida[$j][$i]->getId()));
                    $saida[$j][$i]->setVotosNegativos($this->getTotalVotosNegativos($saida[$j][$i]->getId()));
                    $saida[$j][$i]->setVoto($this->getVoto($saida[$j][$i]->getId()));
                    $i++;
                }

                return $saida;
            }
        } catch (Exception $ex) {
            echo $ex->getMessage();
        }
        return null;
    }

    public function listarRespotasAoTopico($id) {
        try {
            $stmt = $this->connection->prepare("SELECT * FROM resposta,usuario,configuracoes WHERE top_id=:id AND resposta.usu_id=usuario.usu_id AND configuracoes.Usuario_usu_id=usuario.usu_id ORDER BY resposta.`data`, resposta.`hora`");
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);

            if ($stmt->execute()) {
                $saida = array();
                while ($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
                    $resposta = new Resposta($result);
                    $resposta->setVotosPositivos($this->getTotalVotosPositivos($resposta->getId()));
                    $resposta->setVotosNegativos($this->getTotalVotosNegativos($resposta->getId()));
                    $resposta->setVoto($this->getVoto($resposta->getId()));
                    $saida[] = $resposta;
                }
                return $saida;
            }
        } catch (Exception $ex) {
            echo $ex->getMessage();
        }
        return null;
    }

    public function removerResposta($id) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('SELECT usu_id FROM resposta WHERE resp_id=:id');
                $stmt->bindParam(':id', $id, PDO::PARAM_INT);
                $execute = $stmt->execute();
                if($execute){
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    if ((int) $fecth['usu_id'] == (int) $this->iduser) {
                        $stmt = $this->connection->prepare('DELETE FROM resposta WHERE resp_id=:id;');
                        $stmt->bindParam(':id', $id, PDO::PARAM_INT);
                        return $stmt->execute();
                    }
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function novaRespostaAoTopico($id, $texto) {
        if ($this->iduser == -1) {
            return false;
        }

        try {
            $stmt = $this->connection->prepare('INSERT INTO resposta (`resp_texto`, `top_id`, `usu_id`,`data`,`hora`) VALUES (:texto,:idtop,:id,current_date(), current_time());');
            $stmt->bindParam(':idtop', $id, PDO::PARAM_INT);
            $stmt->bindParam(':id', $this->iduser, PDO::PARAM_INT);
            $stmt->bindParam(':texto', $texto, PDO::PARAM_STR);
            return $stmt->execute();
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return false;
    }

    public function getTopico($id) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('SELECT * FROM topico,usuario,configuracoes WHERE top_id=:id AND topico.usu_id=usuario.usu_id AND configuracoes.Usuario_usu_id=usuario.usu_id');
                $stmt->bindParam(':id', $id, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    return new Topico($stmt->fetch(PDO::FETCH_ASSOC));
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function responderTopico($idtopico, $resp_texto) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('call novaResposta(:iduser, :idtop, :resp)');
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':idtop', $idtopico, PDO::PARAM_INT);
                $stmt->bindParam(':resp', $resp_texto, PDO::PARAM_STR);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $fecth['sucess'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function novoTopico($assunto, $texto, $particular, $anonimo, $report, $idturma) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('call novoTopico(:iduser, :topassunto, :toptexto, :idturma, :particular, :anonimo, :report);');
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':topassunto', $assunto, PDO::PARAM_STR);
                $stmt->bindParam(':toptexto', $texto, PDO::PARAM_STR);
                $stmt->bindParam(':idturma', $idturma, $idturma != null ? PDO::PARAM_INT : PDO::PARAM_NULL);
                $stmt->bindParam(':particular', $particular, PDO::PARAM_BOOL);
                $stmt->bindParam(':anonimo', $anonimo, PDO::PARAM_BOOL);
                $stmt->bindParam(':report', $report, PDO::PARAM_BOOL);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $fecth['sucess'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function getDataFormatadaCadastro() {
        if ($this->iduser == -1) {
            return null;
        }
        return date("d/m/Y", strtotime($this->datacadastro));
    }

    public function getDataCadastro() {
        if ($this->iduser == -1) {
            return null;
        }
        return $this->datacadastro;
    }

    public function getNickname() {
        if ($this->iduser == -1) {
            return null;
        }
        return $this->nickname;
    }

    public function getFirstname() {
        if ($this->iduser == -1) {
            return null;
        }
        return $this->firstname;
    }

    public function getLastname() {
        if ($this->iduser == -1) {
            return null;
        }
        return $this->lastname;
    }

    public function getTotalArquivos() {
        try {
            $stmt = $this->connection->prepare('SELECT COUNT(*) AS total FROM arquivo WHERE arquivo.idusuario=:iduser');
            $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);

            if ($stmt->execute() && $stmt->rowCount() == 1) {
                $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                return $fecth['total'];
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return '#ERROR';
    }

    public function listarTodasTurmas() {
        try {
            $stmt = $this->connection->prepare("SELECT * FROM turma");
            if ($stmt->execute()) {
                $saida = array();
                while ($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
                    $saida[] = new Turma($result);
                }
                return $saida;
            }
        } catch (Exception $ex) {
            echo $ex->getMessage();
            return null;
        }
    }

    public function ehAluno() {
        try {
            $stmt = $this->connection->prepare('call verificarEhAluno(:iduser)');
            $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
            $execute = $stmt->execute(); 
            if ($execute && $stmt->rowCount() == 1) {
                $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                return ($fecth['sucess'] == '1');
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return '#ERROR';
    }

    public function ehProfessor() {
        return !$this->ehAluno();
    }

    public function getAluno() {
        return new Aluno($this->iduser);
    }

    public function getProfessor() {
        return new Professor($this->iduser);
    }

    public function getImage() {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('SELECT Path_imguser FROM configuracoes where Usuario_usu_id=:iduser;');
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $fecth['Path_imguser'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return null;
    }

    public function verificarPermissaoPseudocodigoAdd($id_structure) {
        if ($this->iduser == -1) {
            return null;
        }
        try {
            $stmt = $this->connection->prepare('call verificarPermissaoPseudocodigoAdd(:iduser, :idstructure);');
            $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
            $stmt->bindParam(':idstructure', $id_structure, PDO::PARAM_INT);
            if ($stmt->execute() && $stmt->rowCount() == 1) {
                $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                return $fecth['sucess'];
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return null;
    }

    public function verificarPermissaoPseudocodigoSearch($id_structure) {
        if ($this->iduser == -1) {
            return null;
        }
        try {
            $stmt = $this->connection->prepare('call verificarPermissaoPseudocodigoSearch(:iduser, :idstructure);');
            $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
            $stmt->bindParam(':idstructure', $id_structure, PDO::PARAM_INT);
            if ($stmt->execute() && $stmt->rowCount() == 1) {
                $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                return $fecth['sucess'];
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return null;
    }

    public function verificarPermissaoPseudocodigoRem($id_structure) {
        if ($this->iduser == -1) {
            return null;
        }
        try {
            $stmt = $this->connection->prepare('call verificarPermissaoPseudocodigoRem(:iduser, :idstructure);');
            $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
            $stmt->bindParam(':idstructure', $id_structure, PDO::PARAM_INT);
            if ($stmt->execute() && $stmt->rowCount() == 1) {
                $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                return $fecth['sucess'];
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return null;
    }

    public function getTotalRespostaTopico($id) {
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare('SELECT count(resp_id) AS total FROM resposta WHERE top_id=:id');
                $stmt->bindParam(':id', $id, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return (int) $fecth['total'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return 0;
    }

    public function updateResposta($id, $text) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('UPDATE resposta SET `resp_texto`=:texto, `data`=current_date(), `hora`=current_time() WHERE `resp_id`=:id AND `usu_id`=:iduser');
                $stmt->bindParam(':id', $id, PDO::PARAM_INT);
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
                $stmt->bindParam(':texto', $text, PDO::PARAM_STR);
                return $stmt->execute();
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function listarTopicosPublicos($ini, $fim) {
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare('SELECT * FROM topico,usuario WHERE topico.usu_id = usuario.usu_id and visible_only_by_class=false ORDER BY data DESC,hora DESC LIMIT :ini,:fim ');
                $stmt->bindParam(':ini', $ini, PDO::PARAM_INT);
                $stmt->bindParam(':fim', $fim, PDO::PARAM_INT);
                if ($stmt->execute()) {
                    $lista = array();
                    while ($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
                        $lista[] = new Topico($result);
                    }
                    return $lista;
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return null;
    }

    public function buscarTopicosPublicos($ini, $fim, $chave) {
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare("SELECT * FROM topico,usuario WHERE usuario.usu_id = topico.usu_id and topico.top_assunto like '%" . $chave . "%' AND topico.visible_only_by_class=false  ORDER BY data DESC,hora DESC LIMIT :ini,:fim ");
                $stmt->bindParam(':ini', $ini, PDO::PARAM_INT);
                $stmt->bindParam(':fim', $fim, PDO::PARAM_INT);
                if ($stmt->execute()) {
                    $lista = array();
                    while ($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
                        $lista[] = new Topico($result);
                    }
                    return $lista;
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return null;
    }

    public function getTotalRespostaDoUsuario() {
        $total = 0;
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare('SELECT count(resp_id) AS total FROM resposta WHERE usu_id=:id');
                $stmt->bindParam(':id', $this->iduser, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    $total = (int) $fecth['total'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return $total;
    }

    public function getTotalTopicosDoUsuario() {
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare('SELECT count(top_id) AS total FROM topico WHERE usu_id=:id');
                $stmt->bindParam(':id', $this->iduser, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return (int) $fecth['total'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return 0;
    }

    public function getTotalTopicosPublicos() {
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare('SELECT count(top_id) AS total FROM topico WHERE visible_only_by_class=false');
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return (int) $fecth['total'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return 0;
    }

    public function listarTopicosDoUsuario($ini, $fim) {
        if ($this->iduser !== -1) {
            try {
                $stmt = $this->connection->prepare('SELECT * FROM topico WHERE topico.usu_id = :id ORDER BY data DESC,hora DESC LIMIT :ini,:fim ');
                $stmt->bindParam(':ini', $ini, PDO::PARAM_INT);
                $stmt->bindParam(':fim', $fim, PDO::PARAM_INT);
                $stmt->bindParam(':id', $this->iduser, PDO::PARAM_INT);
                if ($stmt->execute()) {
                    $lista = array();
                    while ($result = $stmt->fetch(PDO::FETCH_ASSOC)) {
                        $lista[] = new Topico($result);
                    }
                    return $lista;
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return null;
    }

    public function listarEstruturas() {
        try {
            $stmt = $this->connection->prepare('select * from estrutura_de_dado');
            if ($stmt->execute()) {
                $lista = array();
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
                    $lista[] = new EstruturaDeDados($row);
                }
                return $lista;
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return null;
    }

    public function deletarArquivo($id) {
        try {
            $stmt = $this->connection->prepare('delete FROM arquivo WHERE `arq_id`=:idarq AND `idusuario`=:iduser');
            $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
            $stmt->bindParam(':idarq', $id, PDO::PARAM_INT);
            return $stmt->execute();
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return false;
    }

    public function listarArquivos() {

        try {
            $stmt = $this->connection->prepare('select * from arquivo WHERE idusuario=:iduser');
            $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_INT);
            if ($stmt->execute()) {
                
                $lista = array();
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
                    $lista[] = new Arquivo($row);
                }
                return $lista;
            }
        } catch (PDOException $e) {
            echo $e->getMessage();
        }
        return null;
    }

    public function getTotalVotosPositivos($idresposta) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('SELECT count(*) as total FROM voto WHERE `Resposta_resp_id`=:idresp AND valor=true');
                $stmt->bindParam(':idresp', $idresposta, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $fecth['total'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function getTotalVotosNegativos($idresposta) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('SELECT count(*) as total FROM voto WHERE `Resposta_resp_id`=:idresp AND valor=false');
                $stmt->bindParam(':idresp', $idresposta, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $fecth['total'];
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function setVoto($idresposta, $valor) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('call voteResposta(:idresp,:valor,:iduser)');
                $stmt->bindParam(':idresp', $idresposta, PDO::PARAM_INT);
                $stmt->bindParam(':valor', $valor, PDO::PARAM_INT);
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_BOOL);
                return $stmt->execute();
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function getVoto($idresposta) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->connection->prepare('SELECT valor FROM voto WHERE `Usuario_usu_id`=:iduser AND `Resposta_resp_id`=:idresp');
                $stmt->bindParam(':idresp', $idresposta, PDO::PARAM_INT);
                $stmt->bindParam(':iduser', $this->iduser, PDO::PARAM_BOOL);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    $fecth = $stmt->fetch(PDO::FETCH_ASSOC);
                    return $fecth['valor'] == '1';
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return null;
    }

    public function getTurmaById($idturma) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->getConnection()->prepare('SELECT * FROM turma WHERE `idTurma`=:idturma');
                $stmt->bindParam(':idturma', $idturma, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    return new Turma($stmt->fetch(PDO::FETCH_ASSOC));
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function getFilteredImage() {
        $imageurl = $this->getImage();
        if ($imageurl != null && $imageurl !== '') {
            return $imageurl . '?=' . rand();
        } else {
            return 'images/userdefault.png';
        }
    }

    public function getInstituicao($idinst) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->getConnection()->prepare('SELECT * FROM instituicao WHERE inst_id=:id');
                $stmt->bindParam(':id', $idinst, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    return new Instituicao($stmt->fetch(PDO::FETCH_ASSOC));
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function getCurso($idcurso) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->getConnection()->prepare('SELECT * FROM curso WHERE cur_id=:id');
                $stmt->bindParam(':id', $idcurso, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    return new Curso($stmt->fetch(PDO::FETCH_ASSOC));
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function getDisciplina($iddisciplina) {
        if ($this->iduser != -1) {
            try {
                $stmt = $this->getConnection()->prepare('SELECT * FROM disciplina WHERE disc_id=:id');
                $stmt->bindParam(':id', $iddisciplina, PDO::PARAM_INT);
                if ($stmt->execute() && $stmt->rowCount() == 1) {
                    return new Disciplina($stmt->fetch(PDO::FETCH_ASSOC));
                }
            } catch (PDOException $e) {
                echo $e->getMessage();
            }
        }
        return false;
    }

    public function listarInstituicoes() {
        $instituicoesstmt = $this->getConnection()->prepare('Select * FROM instituicao');
        $cursosstmt = $this->getConnection()->prepare('Select * FROM curso');
        $disciplinasstmt = $this->getConnection()->prepare('Select * FROM disciplina');
        $turmasstmt = $this->getConnection()->prepare('Select * FROM turma');

        if ($instituicoesstmt->execute() && $cursosstmt->execute() && $disciplinasstmt->execute() && $turmasstmt->execute()) {
            $lista = array();
            $instituicoes = $instituicoesstmt->fetchAll(PDO::FETCH_ASSOC);
            $cursos = $cursosstmt->fetchAll(PDO::FETCH_ASSOC);
            $disciplinas = $disciplinasstmt->fetchAll(PDO::FETCH_ASSOC);
            $turmas = $turmasstmt->fetchAll(PDO::FETCH_ASSOC);
            foreach ($instituicoes as $key => $instituicao) {
                $inst = new Instituicao($instituicao);
                $inst->fetchTree($cursos, $disciplinas, $turmas);
                $lista[] = $inst;
            }
            return $lista;
        }
        return null;
    }

}
